import { EventState, Nested } from 'reev';
import { Queue, Frame } from 'refr';

declare type Uniform = number | number[];
declare type Attribute = number[];
declare type GL = EventState<{
    /**
     * initial value
     */
    id: string;
    width: number;
    height: number;
    size: [number, number];
    mouse: [number, number];
    count: number;
    vs: string;
    fs: string;
    vert: string;
    frag: string;
    vertex: string;
    fragment: string;
    varying: string;
    int: PrecisionMode;
    float: PrecisionMode;
    sampler2D: PrecisionMode;
    samplerCube: PrecisionMode;
    lastActiveUnit: number;
    /**
     * core state
     */
    gl: any;
    pg: any;
    el: any;
    queue: Queue;
    frame: Frame;
    target: any;
    stride: Nested<number>;
    location: Nested<any>;
    activeUnit: Nested<number>;
    default: any;
    /**
     * events
     */
    ref?: any;
    init(varying?: string[]): void;
    mount(): void;
    clean(): void;
    render(): void;
    mousemove(e: Event): void;
    resize(e?: Event, width?: number, height?: number): void;
    load(e?: Event, image?: HTMLImageElement): void;
    /**
     * setter
     */
    uniform(key: string, value: Uniform): GL;
    uniform(target: {
        [key: string]: Uniform;
    }): GL;
    texture(key: string, value: string): GL;
    texture(target: {
        [key: string]: string;
    }): GL;
    attribute(key: string, value: Attribute, iboValue?: Attribute): GL;
    attribute(target: {
        [key: string]: Attribute;
    }): GL;
    /**
     * short hands
     */
    clear(key?: GLClearMode): void;
    viewport(size?: [number, number]): void;
    drawArrays(key?: GLDrawMode): void;
    drawElements(key?: GLDrawMode): void;
}>;
declare type PrecisionMode = 'highp' | 'mediump' | 'lowp';
declare type GLClearMode = 'COLOR_BUFFER_BIT' | 'DEPTH_BUFFER_BIT' | 'STENCIL_BUFFER_BIT';
declare type GLDrawMode = 'POINTS' | 'LINE_STRIP' | 'LINE_LOOP' | 'LINES' | 'TRIANGLE_STRIP' | 'TRIANGLE_FAN' | 'TRIANGLES';

export { Attribute as A, GL as G, PrecisionMode as P, Uniform as U, GLClearMode as a, GLDrawMode as b };
